#pragma once
#ifndef __InputManager_H__
#define __InputManager_H__

class InputManager
{
public:
	InputManager();
	~InputManager();

	void RegisterListener(InputListener * Listener);
	void UnregisterListener(InputListener * Listener);

	void RegisterPointerListener(PointerInputListener * PointerListener);
	void UnregisterPointerListener(PointerInputListener * PointerListener);

	void SetMouseCursorVisibility(bool Visible);
	bool IsMouseCursorVisible();

	Vector2n GetWindowDimensions();

	const Vector2n GetMouseCursorPosition() const;		// TODO: Replace with a more proper list of "cursors"

	static void GLFWCALL ProcessWindowSize(int WindowWidth, int WindowHeight);

	static void GLFWCALL ProcessKey(int Key, int Action);
	static void GLFWCALL ProcessChar(int Character, int Action);
	static void GLFWCALL ProcessMouseButton(int MouseButton, int Action);
	static void GLFWCALL ProcessMousePos(int MousePositionX, int MousePositionY);
	static void GLFWCALL ProcessMouseWheel(int MouseWheelPosition);
#ifdef _GLFW_DMITRI_WINDOWS_TOUCH_ENABLED
	static void GLFWCALL ProcessTouchButton(int TouchButton, int Action);
	static void GLFWCALL ProcessTouchPos(int TouchButton, int TouchPositionX, int TouchPositionY, int Action);
#endif // _GLFW_DMITRI_WINDOWS_TOUCH_ENABLED

	void ProcessTimePassed(const double TimePassed);

	struct InputId {
		uint16 Device;
		uint16 Id;

		InputId(uint16 Device, uint16 Id) {
			this->Device = Device;
			this->Id = Id;
		}

		bool operator <(const InputId & Other) const {
			uint32 ThisValue = (static_cast<uint32>(this->Device) << 16) | (this->Id);
			uint32 OtherValue = (static_cast<uint32>(Other.Device) << 16) | (Other.Id);
			return ThisValue < OtherValue;
		}
	};

	typedef uint16 VirtualInputId;

private:
	InputManager(const InputManager &);
	InputManager & operator =(const InputManager &);

	void SetGlfwCallbacks();
	void RemoveGlfwCallbacks();

	TypingPointer										m_TypingPointer;	// FUTURE: For now, assume there is some sort of typing device always present, but later make it dynamically present (maybe?)
	MousePointer										m_MousePointer;		// FUTURE: For now, assume the mouse is always present, but later make it dynamically present
	std::map<uint8, std::unique_ptr<TouchPointer>>		m_TouchPointers;

	std::vector<InputListener *>			m_Listeners;
	std::vector<PointerInputListener *>		m_PointerListeners;

	Vector2n			m_MouseCursorPosition;		// TODO: Replace with a more proper list of "cursors"
	volatile bool		m_MouseCursorVisible;
	volatile bool		m_MouseIgnorePositionAlways;
	volatile bool		m_MouseIgnorePositionOnce;

	Vector2n			m_WindowDimensions;

	static InputManager *		m_pInstance;
};

#endif // __InputManager_H__
