#pragma once
#ifndef __Main_H__
#define __Main_H__

#if defined(_MSC_VER)
#	pragma warning( disable : 4351 )
#endif

#include <iostream>
#include <fstream>
#include <sstream>
#include <set>
#include <map>
#include <list>
#include <cmath>
#include <string>
#include <vector>
#include <cassert>
#include <stdio.h>
#include <string.h>
#include <array>
#include <memory>

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>		// Needed for creating a process to compile projects
#endif // WIN32

#include <GL/glfw.h>

#ifdef WIN32
	typedef signed __int8		sint8;
	typedef signed __int16		sint16;
	typedef signed __int32		sint32;
	typedef signed __int64		sint64;
	typedef unsigned __int8		uint8;
	typedef unsigned __int16	uint16;
	typedef unsigned __int32	uint32;
	typedef unsigned __int64	uint64;
//#elif defined(__APPLE__) && defined(__MACH__)
#else		// For __APPLE__ and Linux
	typedef int8_t				sint8;
	typedef int16_t				sint16;
	typedef int32_t				sint32;
	typedef int64_t				sint64;
	typedef uint8_t				uint8;
	typedef uint16_t			uint16;
	typedef uint32_t			uint32;
	typedef uint64_t			uint64;
#endif // WIN32

using std::endl;

class Concept;
class Project;
class InputManager;
class InputListener;
class PointerInputListener;
class SystemInputListener;
class ControlModuleMapping;
class ControlModule;
class UnrealCameraModule;
class TypingModule;
class WidgetModule;
class WidgetManager;
class WidgetMapping;
class Widget;
class ButtonWidget;
class MultitouchTestBoxWidget;
class TextFieldWidget;
class ListWidget;
class OpenGLStream;
class Canvas;
class Screen;
class ConceptionScreen;
class MultitouchTestScreen;
class Pointer;
class MousePointer;
class TouchPointer;
class TypingPointer;

typedef std::vector<Concept>::size_type ConceptId;

namespace Input
{
	typedef uint16 InputId;

	class AxisState
	{
	public:
		AxisState(double Position, double Length)
			: m_Position(Position),
			  m_Length(Length)
		{
		}

		double GetPosition() const { return m_Position; }
		double GetLength() const { return m_Length; }

	private:
		double m_Position;
		double m_Length;
	};
};

#include "Vector2.h"
#include "ogl_utils.h"
#include "OpenGLStream.h"
#include "Concept.h"
#include "Project.h"
#include "Input/PointerInputListener.h"
#include "Input/WidgetMapping.h"
#include "Input/Pointers/Pointer.h"
#include "Input/Pointers/MousePointer.h"
#include "Input/Pointers/TouchPointer.h"
#include "Input/Pointers/TypingPointer.h"
#include "Input/InputManager.h"
#include "Input/InputListener.h"
#include "Input/SystemInputListener.h"
#include "Input/ControlModuleMapping.h"
#include "ControlModules/ControlModule.h"
#include "ControlModules/UnrealCameraModule.h"
#include "ControlModules/TypingModule.h"
#include "ControlModules/WidgetModule.h"
#include "Widget/WidgetManager.h"
#include "Widget/Widget.h"
#include "Widget/ButtonWidget.h"
#include "Widget/MultitouchTestBoxWidget.h"
#include "Widget/TextFieldWidget.h"
#include "Widget/ListWidget.h"
#include "Canvas.h"
#include "Screen.h"
#include "ConceptionScreen.h"
#include "MultitouchTestScreen.h"
#include "RenderTools.h"

#ifdef WIN32
#elif defined(__APPLE__) && defined(__MACH__)
#endif

extern InputManager * g_InputManager;

extern TypingModule * g_TypingModuleTEST;		// TODO: Delete this obviously, it's a hack for quick testing purposes
extern Screen		* pMainScreen;				// TODO: Delete this...

extern Project CurrentProject;

#endif // __Main_H__
